rm(list=ls());

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# # Sparrows data -- Section A1

# library(vegan);
library(mnormt);

DATA <- read.table("sparrows-data.csv", sep = ";", dec = ".");
DATA <- DATA[,-1];
DATA1 <- DATA[1:21,];
DATA2 <- DATA[22:49,];

# # A pooled correlation matrix
cormat <- cor(DATA);

# Parameters for Survived
sparrows1.mu <- colMeans(DATA1); # Mean values 
sigma1 <- sqrt(diag(var(DATA1)));  # Variances 
sparrows1.var <- outer(sigma1, sigma1) * cormat; # Variance matrix

# Parameters for Non-Survived 
sparrows2.mu <- colMeans(DATA2); # Mean values 
sigma2 <- sqrt(diag(var(DATA2))); # Variances 
sparrows2.var <- outer(sigma2, sigma2) * cormat; # Variance matrix

generate.sparrows <- function(n1, n2, hypothesis = "null"){
#   n1 ... sample size of the first group (Survived)
#   n2 ... sample size of the second group (Non-Survived)
# 	hypothesis = "null" ("alternative") ... The observations are generated, 
# when the null (alternative) hypothesis holds. 

  n <- n1 + n2;
  
  if(hypothesis == "null"){
    temp1 <- rmnorm(n1, mean = sparrows1.mu, varcov = sparrows1.var);
    temp2 <- rmnorm(n2, mean = sparrows2.mu, varcov = sparrows1.var);
    temp <- rbind(temp1, temp2);
#     
  }else{
    temp1 <- rmnorm(n1, mean = sparrows1.mu, varcov = sparrows1.var);
    temp2 <- rmnorm(n2, mean = sparrows2.mu, varcov = sparrows2.var);
		
    temp <- rbind(temp1, temp2);
  }

# # Standardization of the data
# stand <- function(x){
#   vysl <- (x - mean(x))/sd(x);
#   return(vysl);
# }
#   temp <- apply(temp, 2, stand); 

	return(temp);
}

generate.sparrows(n1=21, n2=28);  # Null hypothesis 
generate.sparrows(n1=21, n2=28, hypothesis="alternative"); # Alternative hypothesis 

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# # Generation of data with outliers -- Section A4

generate.outl <- function(nn){
# 	nn ... vector of the sample sizes (n_1,...,n_K)

  n <- sum(nn);
  K <- length(nn);
  p <- 2; # percentage of the outliers 
  DATA <- numeric();
	
  for(i in 1:K){
    temp1 <- rmnorm(nn[i], varcov = diag(p) + matrix(1, p, p));
    temp2 <- rmnorm(nn[i], varcov = 100*diag(p) + matrix(1, p, p));
		
#     outliers
    qqq <- as.logical(runif(nn[i]) < 0.1);
    temp <- rbind(temp1[!qqq,], temp2[qqq,]);
    DATA <- rbind(DATA, temp);      
  }
  
  return(DATA);
}

generate.outl(nn=c(10,10));

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# # Generation of fish type dat  -- Section A2

# # Parameters of the marginal distributions 
fish <- read.table("fish.csv", sep = ";", dec = ".", header = TRUE);
# # A pool correlatin matrix 
cor.fish <- as.matrix(read.table("correlfish.txt", sep = " ", dec = "."));

# The files "fish.csv" and "correlfish.txt" were kindly provided by prof. Marti J. Anderson, University of Auckland. 


# mean values parameters 
mean.fish <- fish[,c(2,4,6,8)];
mu.fish <- mean.fish;

# variance parameters 
var.fish <- fish[,c(2,4,6,8)+1];
# mu2.fish <- var.fish + mean.fish^2;

# # The parameters for the null hypothesis - Berghan Poin
sigma <- sqrt(var.fish[,1]);
var.fish1 <- outer(sigma, sigma) * cor.fish;

# Dealing with zero variances 
qqq <- !(sigma == 0);
var.f1 <- var.fish1[qqq,qqq];
e <- eigen(var.f1);
var.f12 <- e$vectors%*%diag(sqrt(e$values))%*%t(e$vectors); # var.fish^{1/2} 
mu.f1 <- mu.fish[,1];
mu.f12 <- mu.f1[qqq];
p.f1 <- sum(qqq);
p.fish <- length(mu.f1);
    
generate.fish.perm <- function(nn, hypothesis="null", alfa0=0.25, pp=c(1:length(nn))){
# 	nn ... vector of the sample sizes (n_1,...,n_K)	
# 	alfa0 ... the mixing proportion parameter
# 	pp ... the indexes of the groups from which to generate (Berghan Point, "Hahei", "Home Point", "Leigh")

  n <- sum(nn);
  K <- length(nn);
	
  if(hypothesis == "null"){
    sigma <- sqrt(var.fish[,1]);
#     temp <- matrix(0, n, p.fish);
    qqq <- !(sigma == 0);
    
    vysl <- numeric();
    for(i in 1:K){
			temp <- matrix(0, nn[i], p.fish);
# 			Multivariate normal distribution
      norm.temp <- matrix(rnorm(nn[i]*p.f1), nn[i], p.f1) %*% var.f12 + matrix(mu.f12, nn[i], p.f1, byrow = T);
# 			Taking exponential as the paramaters of the independent Poisson variables			
      temp[,qqq] <- exp(norm.temp);      
      temp <- matrix(rpois(nn[i]*p.fish, lambda = temp), nn[i], p.fish);
			
      temp <- temp[,sample(p.fish)]; # a random permutation of the coordinates 
      vysl <- rbind(vysl, temp);
    }
  }else{
#     
      if(K > 4){ 
        print("For the alternative K <= 4 is required !!!")
        stop();
      }
      
      
      if(length(nn) != length(pp)){
        print("Dimension of nn and pp does not match !!!");
        stop();
      }
      
      sigmaA <- sqrt(var.fish[,pp]);
      mu.simA <- mu.fish[,pp];
      p <- nrow(mu.simA);
      
      K <- length(nn);
  #     vysl <- list();
      
      vysl <- matrix(NA, n, p); 
      i1 <- c(1, 1+cumsum(nn)[-K]);
      i2 <- c(cumsum(nn));
      
      sigma <- sigmaA[,1];
      mu.sim <- mu.simA[,1];
#         browser();
  #       
      qqq1 <- !(sigma == 0);
      var.sim <- outer(sigma, sigma) * cor.fish;
      var.sim1 <- as.matrix(var.sim[qqq1,qqq1]);
      mu.sim1 <- mu.sim[qqq1];
        
      
  #     browser();
      for(i in 1:K){
        temp2 <- matrix(0, nn[i], p);
        temp1 <- matrix(0, nn[i], p);
        temp <- matrix(0, nn[i], p);
				
# 				Parameters of th actual group
        sigma <- sigmaA[,i];
        mu.sim <- mu.simA[,i];
        qqq <- !(sigma == 0);
        var.sim <- outer(sigma, sigma) * cor.fish;
        var.sim2 <- as.matrix(var.sim[qqq,qqq]);
        mu.sim2 <- mu.sim[qqq];
				
        temp1A <- exp(rmnorm(nn[i], mean = mu.sim1, varcov = var.sim1)); # Berghan Point
        temp2A <- exp(rmnorm(nn[i], mean = mu.sim2, varcov = var.sim2)); # actual group
        
        temp2[,qqq] <- temp2A;
        temp1[,qqq1] <- temp1A;
        
# 			Mixing of the Berghan point and the actual group
        mixing <- as.logical(runif(nn[i]) < alfa0);
        temp[mixing,] <- temp2[mixing,] ;
        temp[!mixing,] <- temp1[!mixing,] ;

        temp <- temp[,sample(p)]; # a random permutation of the coordinates 
        vysl[i1[i]:i2[i],] <- matrix(rpois(nn[i]*p, lambda = temp), nn[i], p); # poisson distribution
      }
  }  
  return(vysl);
}

generate.fish.perm(nn=c(10,10,10,10), hypothesis="null");
generate.fish.perm(nn=c(10,10,10,10), hypothesis="alt");

# Berghan Point and Hahei (as used in Section A2.1)
generate.fish.perm(nn=c(100,100), hypothesis="alt", pp=c(1,4), alfa0=0.1);

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# # Generation of corals type dat  -- Section A3

corals <- read.table("corals.csv", sep = ";", dec = ".", header = TRUE);
p.corals <- nrow(corals);

# # The files "fish.csv" and "correlfish.txt" were kindly provided by prof. Marti J. Anderson, University of Auckland. 

# # Alterantively, the estimate of the parameters can be retrieved as 
# load("tikus.RData");
# corals.data <- tikus$abund;
# corals.data <- corals.data[,as.logical(colSums(corals.data) > 0)];
# 
# # Proportion of non-zeroes
# prop.cor <- apply((corals.data > 0), 2, function(x) tapply(x, tikus$x[,1], mean));
# 
# # Mean coverage for the nonzero observations
# mean.cor <- apply(corals.data, 2, function(x) tapply(x, tikus$x[,1], sum))/apply((corals.data > 0), 2, function(x) tapply(x, tikus$x[,1], sum));
# mean.cor[prop.cor == 0] <- 0;
# 
# # Variances of the nonzero observations
# fce <- function(x){
# 	nx <- length(unique(x[x > 0])); # Number of different nonzero observations
# 	if(nx >= 2){
# 		return(var(x[x > 0]));
# 	}else{
# 		return(0);
# 	}
# }
# 
# var.cor <- apply(corals.data, 2, function(x) tapply(x, tikus$x[,1],fce));

# Warning: Although most of the estimates of the parameters coincide 
# with the estimates from the file "corals.csv", this is not true for 
# all the estimates.,, 


generate.corals <- function(nn, hypothesis="null"){
  n <- sum(nn);

	
# # 	Null hypothesis 
  if(hypothesis == "null"){
#   Setting the parameters of the multivariate normal distribution
    sigma <- sqrt(corals[,c(19)]);
    mu <- corals[,c(18)];
    prop <- corals[,c(17)];
#       Generation of the random variables
    temp <- matrix(rnorm(n*p.corals), n, p.corals) * matrix(sigma, n, p.corals, byrow = T) + matrix(mu, n, p.corals, byrow = T);		
#       Setting an appropriate proportion of the generated data to zero  
    qqq <- as.logical(matrix(runif(n*p.corals), n, p.corals) > matrix(prop, n, p.corals, byrow = T));
    temp[qqq] <- 0;
		
# 		Taking positive part and truncation to the nearest integers
    vysl <- trunc(pmax(temp,0));
		
# # 		Alternative hypothesis
  }else{
#     
      K <- length(nn);
			if(K > 6){ 
        print("For the alternative K <= 6 is required !!!")
        stop();
      }
      
      i.var <- c(4,7,10,13,16,19); # indexes of variance parameters
      i.mu <- i.var - 1; # indexes of mean parameters
      i.prop <- i.var - 2; # indexes of nonzero proportion 
      
      sigma1 <- sqrt(corals[,i.var]);
      mu.sim1 <- corals[,i.mu];
      prop.sim1 <- corals[,i.prop];
      
      p <- nrow(mu.sim1);
    
      vysl <- matrix(NA, n, p); 
      i1 <- c(1, 1+cumsum(nn)[-K]);
      i2 <- c(cumsum(nn));
      
      for(i in 1:K){
        temp <- matrix(0, nn[i], p.corals);
#   Setting the parameters of the multivariate normal distribution
        sigma <- sigma1[,i];
        mu <- mu.sim1[,i];
        prop  <- prop.sim1[,i];

# 				browser();
#       Generation of the random variables
        temp <- matrix(rnorm(nn[i]*p.corals), nn[i], p.corals) * matrix(sigma, nn[i], p.corals, byrow = T) + matrix(mu, nn[i], p.corals, byrow = T);
#       Setting an appropriate proportion of the generated data to zero  
        qqq <- as.logical(matrix(runif(nn[i]*p.corals), nn[i], p.corals) > matrix(prop, nn[i], p.corals, byrow = T));
        temp[qqq] <- 0;
        
# 		Taking positive part and truncation to the nearest integers
        vysl[i1[i]:i2[i],] <- trunc(pmax(temp,0))
      }
  }
  
#   vysl <- sqrt(vysl); # sqrt-root transformation
  return(vysl);
}

generate.corals(nn=rep(10,6));
generate.corals(nn=rep(10,6), hypothesis="alternative");
